'use strict';

var reader = require('../lib/reader.js');
var common = require('./common.js');
var verifyStoreReader = require('./verify-store-reader.js');

describe('CarReader fromBytes()', () => {
  it('complete', async () => {
    const reader$1 = await reader['default'].fromBytes(common.carBytes);
    await verifyStoreReader.verifyRoots(reader$1);
    await verifyStoreReader.verifyHas(reader$1);
    await verifyStoreReader.verifyGet(reader$1);
    await verifyStoreReader.verifyBlocks(reader$1.blocks());
    await verifyStoreReader.verifyCids(reader$1.cids());
    common.assert.strictEqual(reader$1.version, 1);
  });
  it('complete (get before has) switch', async () => {
    const reader$1 = await reader['default'].fromBytes(common.carBytes);
    await verifyStoreReader.verifyRoots(reader$1);
    await verifyStoreReader.verifyGet(reader$1);
    await verifyStoreReader.verifyHas(reader$1);
    await verifyStoreReader.verifyBlocks(reader$1.blocks());
    await verifyStoreReader.verifyCids(reader$1.cids());
  });
  it('bad argument', async () => {
    for (const arg of [
        true,
        false,
        null,
        undefined,
        'string',
        100,
        { obj: 'nope' }
      ]) {
      await common.assert.isRejected(reader['default'].fromBytes(arg));
    }
  });
  it('decode error - truncated', async () => {
    await common.assert.isRejected(reader['default'].fromBytes(common.carBytes.slice(0, common.carBytes.length - 10)), {
      name: 'Error',
      message: 'Unexpected end of data'
    });
  });
});
describe('CarReader fromIterable()', () => {
  it('complete (single chunk)', async () => {
    const reader$1 = await reader['default'].fromIterable(common.makeIterable(common.carBytes, common.carBytes.length));
    await verifyStoreReader.verifyRoots(reader$1);
    await verifyStoreReader.verifyHas(reader$1);
    await verifyStoreReader.verifyGet(reader$1);
    await verifyStoreReader.verifyBlocks(reader$1.blocks());
    await verifyStoreReader.verifyCids(reader$1.cids());
  });
  it('complete (101-byte chunks)', async () => {
    const reader$1 = await reader['default'].fromIterable(common.makeIterable(common.carBytes, 101));
    await verifyStoreReader.verifyRoots(reader$1);
    await verifyStoreReader.verifyHas(reader$1);
    await verifyStoreReader.verifyGet(reader$1);
    await verifyStoreReader.verifyBlocks(reader$1.blocks());
    await verifyStoreReader.verifyCids(reader$1.cids());
  });
  it('complete (64-byte chunks)', async () => {
    const reader$1 = await reader['default'].fromIterable(common.makeIterable(common.carBytes, 64));
    await verifyStoreReader.verifyRoots(reader$1);
    await verifyStoreReader.verifyHas(reader$1);
    await verifyStoreReader.verifyGet(reader$1);
    await verifyStoreReader.verifyBlocks(reader$1.blocks());
    await verifyStoreReader.verifyCids(reader$1.cids());
  });
  it('complete (32-byte chunks)', async () => {
    const reader$1 = await reader['default'].fromIterable(common.makeIterable(common.carBytes, 32));
    await verifyStoreReader.verifyRoots(reader$1);
    await verifyStoreReader.verifyHas(reader$1);
    await verifyStoreReader.verifyGet(reader$1);
    await verifyStoreReader.verifyBlocks(reader$1.blocks());
    await verifyStoreReader.verifyCids(reader$1.cids());
  });
  it('bad argument', async () => {
    for (const arg of [
        new Uint8Array(0),
        true,
        false,
        null,
        undefined,
        'string',
        100,
        { obj: 'nope' }
      ]) {
      await common.assert.isRejected(reader['default'].fromIterable(arg));
    }
  });
  it('decode error - truncated', async () => {
    await common.assert.isRejected(reader['default'].fromIterable(common.makeIterable(common.carBytes.slice(0, common.carBytes.length - 10), 64)), {
      name: 'Error',
      message: 'Unexpected end of data'
    });
  });
});
